package org.jibx.xsd2jibx;

import java.util.ArrayList;
import java.util.HashMap;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ws.jaxme.js.JavaMethod;
import org.apache.ws.jaxme.js.JavaQName;
import org.apache.ws.jaxme.js.JavaQNameImpl;
import org.apache.ws.jaxme.js.JavaSource;
import org.apache.ws.jaxme.js.JavaSource.Protection;

public class JSource {
	
	public static boolean CREATE_ALL_CONSTRUCTOR = true;
	public static boolean CREATE_REQUIRED_CONSTRUCTOR = true;
	
	private static final Log log = LogFactory.getLog(JSource.class);

	public static final JavaSource.Protection PUBLIC = JavaSource.PUBLIC;
	public static final JavaSource.Protection PRIVATE = JavaSource.PRIVATE;
	public static final JavaSource.Protection DEFAULT = JavaSource.DEFAULT_PROTECTION;
	public static final JavaSource.Protection PROTECTED = JavaSource.PROTECTED;
    
    private static HashMap s_collectionTypeMap;
    static {
        s_collectionTypeMap = new HashMap();
        s_collectionTypeMap.put("int", JavaQNameImpl.getInstance("java.lang.Integer"));
        s_collectionTypeMap.put("boolean", JavaQNameImpl.getInstance("java.lang.Boolean"));
        s_collectionTypeMap.put("byte", JavaQNameImpl.getInstance("java.lang.Byte"));
        s_collectionTypeMap.put("char", JavaQNameImpl.getInstance("java.lang.Character"));
        s_collectionTypeMap.put("short", JavaQNameImpl.getInstance("java.lang.Short"));
        s_collectionTypeMap.put("long", JavaQNameImpl.getInstance("java.lang.Long"));
        s_collectionTypeMap.put("float", JavaQNameImpl.getInstance("java.lang.Float"));
        s_collectionTypeMap.put("double", JavaQNameImpl.getInstance("java.lang.Double"));
    }

	private JavaSource source;

	private ArrayList fields = new ArrayList();

	JSource(JavaSource source) {
		this.source = source;
	}

	public void addExtends(JQName qname) {
		source.addExtends(qname.getJavaQName());
	}

	public void newBeanProperty(JQName qname, String name, boolean required) {
		if (qname == null) {
			log.error("qname == null, name: " + name);
		} else {
			source.newBeanProperty(qname.getJavaQName(), name);
		}
	}

	private JConstructor newJConstructor(Protection protection) {
		return new JConstructor(source.newJavaConstructor(protection));
	}

	/**
	 * @return How many of the fields added are required.
	 */
	void addElement(XsdElement element) {
		if (element.isCollection()) {
			
			// TODO fix ArrayList workaround
			// create an array and size(), get(), add() methods
			// THIS ISN'T WORKING, below is a workaround.
			//sourceFile.addLine( ArrayList.class, " " + eName + "List = new ", ArrayList.class, "();" );
			source.addRawJavaSource("  protected ArrayList " + element.getFieldList() + " = new ArrayList();");
			// Problem: adds multiple import statements for ArrayList.
			// JavaSource needs an isImport() method.
			source.addImport(ArrayList.class);
            
            // determine type to be used for values
            String type = element.getFieldQName().getClassName();
            JavaQName jqname = (JavaQName)s_collectionTypeMap.get(type);
            if (jqname == null) {
                jqname = element.getFieldQName().getJavaQName();
            } else {
                type = jqname.getClassName();
            }

			// create addMethod
			JavaMethod jm = source.newJavaMethod(element.getAddMethod(), JQName.VOID.getJavaQName(), PUBLIC);
			jm.addParam(jqname, element.getFieldName());
			jm.addLine(element.getFieldList() + ".add(" + element.getFieldName() + ");");

			// create getMethod
			jm = source.newJavaMethod(element.getGetMethod(), jqname, PUBLIC);
			jm.addParam(JQName.INT.getJavaQName(), "index");
			jm.addLine("return (" + type + ")" + element.getFieldList() + ".get( index );");

			//create size method
			jm = source.newJavaMethod(element.getSizeMethod(), JQName.INT.getJavaQName(), JavaSource.PUBLIC);
			jm.addLine("return " + element.getFieldList() + ".size();");
		} else { // it is not a collection
			newBeanProperty(element);
		}
	}

	void newBeanProperty(XsdFeature s) {

		try {
//			log.debug("addBeanProperty, " + s);
			//source.newJavaField( s.getFieldName( ), s.getFieldQName(
			// ).getJavaQName(), PROTECTED );
			newField(s);
			JavaMethod getMethod = source.newJavaMethod(s.getGetMethod(), s
					.getFieldQName().getJavaQName(), JavaSource.PUBLIC);
			getMethod.addLine("return this.", s.getFieldName(), ";");
			JavaMethod setMethod = source.newJavaMethod(s.getSetMethod(),
					JQName.VOID.getJavaQName(), JavaSource.PUBLIC);
			JQName fieldType = s.getFieldQName();
			if (fieldType == null) {
				log.warn("fieldType == null, defaulting to String: "
						+ s.getFieldName());
				fieldType = JQName.STRING;
			}
			setMethod.addParam(fieldType.getJavaQName(), s.getFieldName());
			setMethod.addLine("this.", s.getFieldName(), " = ", s
					.getFieldName(), ";");
		} catch (IllegalStateException e) {
			log.error("field probably exists already", e);
		}

	}

	//    public JavaField[] getFields() {
	//        return source.getFields();
	//    }

	private ArrayList getRequiredFields() {
		ArrayList requiredFields = new ArrayList();

		for (int i = 0, n = fields.size(); i < n; i++) {
			JField field = (JField) fields.get(i);
			if (field.isRequired()) {
				requiredFields.add(field);
			}
		}
		return requiredFields;
	}

	public void createConstructors() {
		
//		ArrayList requiredFields = getRequiredFields();
//		int requiredFieldSize = fields.size();
//		if (requiredFieldSize == 0) {
//			newJConstructor(JSource.PUBLIC);
//		} else {
//			// default constructor for JiBX, not sure if this is required now
//			newJConstructor(JSource.PROTECTED);
//
//			JConstructor constructorAll = newJConstructor(JSource.PUBLIC);
//			//log.debug( "number of java fields: " + fields.length );
//			for (int i = 0; i < requiredFieldSize; i++) {
//				JavaField field = ((JField) fields.get(i)).getJavaField();
//				String fieldName = field.getName();
//				constructorAll.addParam(new JQName(field.getType()), fieldName);
//			}
//
//		}

	}

	private void newField(XsdFeature s) {
		JavaQName qname = s.getFieldQName().getJavaQName();
		JField field = new JField(source.newJavaField(s.getFieldName(), qname, PROTECTED), s.isRequired());
		fields.add(field);
	}
	
//	TODO  refractor, was a copy of addElement()
	public void addListWrapper(XsdElement element, XsdElement wrappedElement) {
		
		source.addRawJavaSource("  protected ArrayList " + element.getFieldList() + " = new ArrayList();");
		source.addImport(ArrayList.class);

		// create addMethod
		JavaMethod jm = source.newJavaMethod(element.getAddMethod(), JQName.VOID.getJavaQName(), PUBLIC);
		jm.addParam(wrappedElement.getFieldQName().getJavaQName(), element.getFieldName());
		jm.addLine(element.getFieldList() + ".add( " + element.getFieldName() + " );");

		// create getMethod
		jm = source.newJavaMethod(element.getGetMethod(), wrappedElement.getFieldQName().getJavaQName(), PUBLIC);
		jm.addParam(JQName.INT.getJavaQName(), "index");
		jm.addLine("return (" + wrappedElement.getFieldQName().getClassName() + ") " + element.getFieldList() + ".get( index );");

		//create size method
		jm = source.newJavaMethod(element.getSizeMethod(), JQName.INT.getJavaQName(), JavaSource.PUBLIC);
		jm.addLine("return " + element.getFieldList() + ".size();");
	}

    
    public String toString(){
        return "JSource[ name: "+ source.getClassName() +" ]";
    }
}